@@ -9,6 +9,7 @@ from __future__ import with_statement
 from __future__ import absolute_import
 from __future__ import print_function
 import argparse
+import html
 import logging
 
 #import lxml.etree
@@ -149,7 +150,7 @@ class JUnitMgr(object):
         self.options = options
         self.file_name = file_name
         self.summary = None
-        self.test_suite = None
+        self._test_suite = None
 
 
     def load(self):
@@ -157,45 +158,90 @@ class JUnitMgr(object):
         :return:
         """
         retobj = JUnitMgr.parseJUnit(file_name=self.file_name, options=self.options)
-        self.test_suite = retobj.get('test_suite')
+        self._test_suite = retobj.get('test_suite')
+        self._fails = retobj.get('failed_tests')
         self.summary = retobj.get('summary')
 
 
     def results(self):
         return self.summary
 
-    def show_summary(self):
+    def show_fails(self):
+        logging.info('*' * 72)
+        logging.info("== Failed Tests ==")
+        n = 0
+        for tc in self._fails:
+            print("{n}. {name}\n\t{message}\n".format(n=n, name=tc.name, message=html.unescape(tc.result.message)))
+            n += 1
+
+    def show_summary(self, display_only_failures=False, pretty=True):
         logging.info('*' * 72)
         logging.info("== SUMMARY ==")
         n = 0
-        for ts in self.test_suite:
-            print("== {} ==".format(n))
-            print("o name  : {}".format(ts.name))
-            print("o tag   : {}".format(ts._tag))
-            print("o time  : {}".format(ts.time))
-            print("o tests : {}".format(self.test_suite.tests))
 
-            if hasattr(ts, 'result'):
-                if isinstance(ts.result, Error):
-                    # results['message'] = ts.result.message
-                    print(ts.result.message)
+        for ts in self._test_suite:
+            if display_only_failures:
+                if hasattr(ts, 'result') and (isinstance(ts.result, Failure) or isinstance(ts.result, Error)):
+                    #print("{name}\n\t{message}".format(name=ts.name, message=ts.result.message))
+                    print("{name}\n\t{message}\n".format(name=ts.name, message=html.unescape(ts.result.message)))
+            else:
+                _msg = None
+                _result = None
+                _tm = None
+
+                if hasattr(ts, 'result'):
+
+                    if hasattr(ts, 'time'):
+                        _tm = ": {tm}".format(tm=ts.time)
+
+                    if isinstance(ts.result, Error):
+                        # results['message'] = ts.result.message
+                        #print(ts.result.message)
+                        _msg = ts.result.message
+                        _result = "ERROR"
+
+                    elif isinstance(ts.result, Failure):
+                        # results['message'] = ts.result.message
+                        #print(ts.result.message)
+                        _msg = ts.result.message
+                        _result = "FAIL"
+
+                    elif ts.result is None:
+                        _result = "PASS"
+                        _msg = "PASS"
+
+                if pretty and _msg is not None:
+                    if _msg == "PASS":
+                        print("{name} ... {result} {tm}".format(name=ts.name, result=_result, tm=_tm))
+                    else:
+                        print("{name} ... {result} {tm}\n\t{message}\n".format(name=ts.name, result=_result, tm=_tm, message=html.unescape(_msg)))
+                elif not pretty:
+                    print("== {} ==".format(n))
+                    print("o name  : {}".format(ts.name))
+                    print("o tag   : {}".format(ts._tag))
+                    print("o time  : {}".format(ts.time))
+                    print("o tests : {}".format(self._test_suite.tests))
+
 
-                if isinstance(ts.result, Failure):
-                    # results['message'] = ts.result.message
-                    print(ts.result.message)
             n += 1
 
         print('*' * 72)
         print("== SUMMARY ==")
+        print("name: {}".format(self.summary.get('name')))
+        print("failures: {}".format(self.summary.get('failures')))
+        print("errors: {}".format(self.summary.get('errors')))
+        print("tests: {}".format(self.summary.get('tests')))
+        print("skipped: {}".format(self.summary.get('skipped')))
+        print("time: {}".format(self.summary.get('time')))
         print("Results: {}".format(self.summary))
 
     def test_suite(self):
-        return self.test_suite
+        return self._test_suite
 
 
     def readXML(self, f):
         suite = TestSuite('name')
-        results = {'failures': 0, 'errors': 0, 'tests': 0, 'skipped': 0, 'time': 0, 'message': None}
+        results = {'failures': 0, 'errors': 0, 'tests': 0, 'skipped': 0, 'time': 0 } #, 'message': None}
         p = XMLParser(huge_tree=True)
         tree = parse(f, parser=p)
         testsuite = tree.xpath('//testsuite')
@@ -221,13 +267,13 @@ class JUnitMgr(object):
                     tc.result = Failure()
                     logging.info("== failure found ==")
 
-                if child.tag == "error":
+                elif child.tag == "error":
                     tc.result = Error()
                     tc.result.message = child.text
 
                     logging.info("[DEBUG][child] error: {}\n".format(child.text))
 
-                if child.tag == "skip" or child.tag == 'skipped':
+                elif child.tag == "skip" or child.tag == 'skipped':
                     tc.result = Skipped()
 
             if self.options.get('verbose'):
@@ -283,8 +329,9 @@ class JUnitMgr(object):
             options = {'verbose': False}
 
         suite = TestSuite('name')
-        results = {'failures': 0, 'errors': 0, 'tests': 0, 'skipped': 0, 'time': 0, 'message': None}
+        results = {'failures': 0, 'errors': 0, 'tests': 0, 'skipped': 0, 'time': 0 } #, 'message': None}
         p = XMLParser(encoding='utf-8', huge_tree=huge_tree)
+
         tree = parse(source=file_name, parser=p)
         testsuite = tree.xpath('//testsuite')
         test_cases = tree.xpath('//testcase')
@@ -305,15 +352,21 @@ class JUnitMgr(object):
                     logging.info("[DEBUG][child] {}".format(child))
                     logging.info("[DEBUG][child] tag : {}".format(child.tag))
 
+                src_line = ""
+                if hasattr(child, 'sourceline'):
+                    src_line = "(source line:{src})".format(src=child.sourceline)
+
                 if child.tag == "testcase":
                     pass
                 elif child.tag == "failure":
                     logging.info("== failure found ==")
-                    tc.result = Failure()
+                    tc.result = [ Failure(message="{text} {src}".format(text=child.text, src=src_line)) ]
                 elif child.tag == "error":
-                    tc.result = Error()
+                    tc.result = [ Error(message="{text} {src}".format(text=child.text, src=src_line)) ]
                 elif child.tag == "skip" or child.tag == 'skipped':
-                    tc.result = Skipped()
+                    tc.result = [ Skipped(message="{text} {src}".format(text=child.text, src=src_line)) ]
+                elif child.tag == "system-out" or child.tag == "system-in":
+                    pass
                 else:
                     logging.debug("====== UNKNOWN TAG:{}".format(child.tag))
 
@@ -324,7 +377,6 @@ class JUnitMgr(object):
 
             tc.time = float(t.get('time'))
 
-
             logging.info("DEBUG => {}".format(type(tc)))
             logging.info("DEBUG.dir => {}".format(dir(tc)))
 
@@ -388,15 +440,14 @@ class JUnitMgr(object):
             options = {'verbose': False}
 
         test_suites = None
-        results = {'name': None, 'failures': 0, 'errors': 0, 'tests': 0, 'skipped': 0, 'time': 0, 'message': None}
+        results = {'name': None, 'failures': 0, 'errors': 0, 'tests': 0, 'skipped': 0, 'time': 0 } #, 'message': None}
         try:
             # <class 'junitparser.junitparser.TestSuite'>
             # testSuites = JUnitXml.fromfile(f)
             test_suites = JUnitMgr.parse_xml(file_name, options)
         except Exception as ex:
-            logging.info(ex)
+            logging.critical(ex)
 
-        # logging.info(type(testSuites))
         results['name'] = test_suites.name
         results['errors'] += test_suites.errors
         results['failures'] += test_suites.failures
@@ -407,6 +458,8 @@ class JUnitMgr(object):
         results['tests'] += test_suites.tests
         results['time'] += test_suites.time
 
+        failed_tests = []
+
         for ts in test_suites:
             if options.get('verbose'):
                 logging.info("o name  : {}".format(ts.name))
@@ -416,12 +469,14 @@ class JUnitMgr(object):
 
             if hasattr(ts, 'result'):
                 if isinstance(ts.result, Error):
-                    results['message'] = ts.result.message
+                    #results['message'] = ts.result.message
+                    failed_tests.append(ts)
 
                 if isinstance(ts.result, Failure):
-                    results['message'] = ts.result.message
+                    #results['message'] = ts.result.message
+                    failed_tests.append(ts)
 
-        return {'summary': results, 'test_suite': test_suites}
+        return {'summary': results, 'test_suite': test_suites, 'failed_tests': failed_tests}
 
 
 def parse_args():
